/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_CXT_H
#define SDXI_CXT_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))


/* From Table 32: Context Level 2 Table Entry (CXT_L2_ENT) */
struct  SDXI_CXT_L2_ENT {
    uint64_t  lv01_ptr;               /* 000:000, vl                       */
                                      /* 011:001, rsvd                     */
                                      /* 063:012, lv01_ptr                 */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 33: Context Level 1 Table Entry (CXT_L1_ENT) */
struct  SDXI_CXT_L1_ENT {
    uint64_t  cxt_ctl_ptr;            /* 000:000, vl                       */
                                      /* 001:001, ka                       */
                                      /* 002:002, pv                       */
                                      /* 005:003, rsvd                     */
                                      /* 063:006, cxt_ctl_ptr              */
    uint64_t  akey_ptr;               /* 067:064, akey_sz                  */
                                      /* 075:068, rsvd                     */
                                      /* 127:076, akey_ptr                 */
    uint32_t  misc0;                  /* 147:128, cxt_pasid                */
                                      /* 151:148, max_buffer               */
                                      /* 159:152, rsvd                     */
    uint32_t  opb_000_enb;            /* 191:160,                          */
    uint8_t   rsvd_0[8];              /* 255:192,                          */
} _SDXI_PACKED _SDXI_ALIGN(32);


/* From Table 34: Context Control (CXT_CTL) */
struct  SDXI_CXT_CTL {
    uint64_t  ds_ring_ptr;            /* 000:000, vl                       */
                                      /* 001:001, rsvd                     */
                                      /* 003:002, qos                      */
                                      /* 004:004, se                       */
                                      /* 005:005, csa                      */
                                      /* 063:006, ds_ring_ptr              */
    uint32_t  ds_ring_sz;             /* 095:064,                          */
    uint8_t   rsvd_0[4];              /* 127:096,                          */
    uint64_t  cxt_sts_ptr;            /* 131:128, rsvd                     */
                                      /* 191:132, cxt_sts_ptr              */
    uint64_t  write_index_ptr;        /* 194:192, rsvd                     */
                                      /* 255:195, write_index_ptr          */
    uint8_t   rsvd_1[32];             /* 511:256,                          */
} _SDXI_PACKED _SDXI_ALIGN(64);


/* From Table 35: Context Status (CXT_STS) */
struct  SDXI_CXT_STS {
    uint8_t   state;                  /* 003:000, state                    */
                                      /* 007:004, rsvd                     */
    uint8_t   misc0;                  /* 008:008, rsh                      */
                                      /* 015:009, rsvd                     */
    uint8_t   rsvd0_[6];              /* 063:016,                          */
    uint64_t  read_index;             /* 127:064,                          */
} _SDXI_PACKED _SDXI_ALIGN(16);

#endif /* SDXI_CXT_H */
