/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_ERRLOG_H
#define SDXI_ERRLOG_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))


/* From Table 39: Error Log Header Entry (ERRLOG_HD_ENT) */
struct  SDXI_ERRLOG_HD_ENT {
    uint32_t  opcode;                 /* 000:000, vl = 1                   */
                                      /* 007:001, rsvd                     */
                                      /* 013:008, step                     */
                                      /* 015:014, rsvd                     */
                                      /* 026:016, type = 0x7F7             */
                                      /* 031:027, rsvd                     */
    uint16_t  misc0;                  /* 032:032, cv                       */
                                      /* 033:033, div                      */
                                      /* 034:034, bv                       */
                                      /* 035:035, rsvd                     */
                                      /* 038:036, buf                      */
                                      /* 039:039, rsvd                     */
                                      /* 043:040, sub_step                 */
                                      /* 046:044, re                       */
                                      /* 047:047, rsvd                     */
    uint16_t  cxt_num;                /* 063:048,                          */
    uint64_t  dsc_index;              /* 127:064,                          */
    uint8_t   rsvd_0[28];             /* 351:128,                          */
    uint16_t  err_class;              /* 367:352,                          */
    uint8_t   rsvd_1[2];              /* 383:368,                          */
    uint32_t  vendor[4];              /* 511:384,                          */
} _SDXI_PACKED _SDXI_ALIGN(64);

#endif /* SDXI_ERRLOG_H */
