/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_MMIO_H
#define SDXI_MMIO_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))


/* From Table 92: MMIO_CTL0 (MMIO Offset :0x0_0000) */
struct  SDXI_MMIO_CTL0 {
    uint32_t  field0;                 /* 001:000, fn_gsr                   */
                                      /* 002:002, fn_pasid_vl              */
                                      /* 003:003, rsvd                     */
                                      /* 004:004, fn_err_intr_en           */
                                      /* 007:005, rsvd                     */
                                      /* 027:008, fn_pasid                 */
                                      /* 031:028, rsvd                     */
    uint32_t  fn_grp_id;              /* 063:032,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 93: MMIO_GRP_ENUM (MMIO Offset: 0x0_0008) */
struct  SDXI_MMIO_GRP_ENUM {
    uint8_t   field0;                 /* 000:000, busy                     */
                                      /* 001:001, probe                    */
                                      /* 007:002, rsvd                     */
    uint8_t   rsvd_0[7];              /* 063:008,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 96: MMIO_CAP0 (MMIO Offset: 0x0_0200) */
struct  SDXI_MMIO_CAP0 {
    uint16_t  sfunc;                  /* 015:000,                          */
    uint8_t   field0;                 /* 016:016, vf                       */
                                      /* 018:017, cs_cap                   */
                                      /* 019:019, rsvd                     */
                                      /* 022:020, db_stride                */
                                      /* 023:023, rsvd                     */
    uint8_t   max_ds_ring_sz;         /* 028:024, max_sz                   */
                                      /* 031:029, rsvd                     */
    uint8_t   max_rkey_sz;            /* 035:032, max_sz                   */
                                      /* 039:036, rsvd                     */
    uint8_t   rsvd_0[3];              /* 063:040,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 97: MMIO_CAP1 (MMIO Offset: 0x0_0208) */
struct  SDXI_MMIO_CAP1 {
    uint16_t  field0;                 /* 003:000, max_buffer               */
                                      /* 004:004, rkey_cap                 */
                                      /* 005:005, rm                       */
                                      /* 006:006, mmio64                   */
                                      /* 007:007, rsvd                     */
                                      /* 011:008, max_errlog_sz            */
                                      /* 015:012, max_akey_sz              */
    uint16_t  max_cxt;                /* 031:016,                          */
    uint32_t  opb_000_cap;            /* 063:032,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 98: MMIO_VERSION (MMIO Offset: 0x0_0210) */
struct  SDXI_MMIO_VERSION {
    uint8_t   minor;                  /* 007:000,                          */
    uint8_t   rsvd_0;                 /* 015:008,                          */
    uint8_t   major;                  /* 023:016,                          */
    uint8_t   rsvd_1;                 /* 031:024,                          */
    uint8_t   rsvd_2[4];              /* 063:032,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 99: MMIO_CXT_L2 (MMIO Offset: 0x1_0000^) */
struct  SDXI_MMIO_CXT_L2 {
    uint64_t  lv02_ptr;               /* 011:000, rsvd                     */
                                      /* 063:012, lv02_ptr                 */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 910: MMIO_RKEY (MMIO Offset: 0x1_0100) */
struct  SDXI_MMIO_RKEY {
    uint64_t  ptr;                    /* 000:000, en                       */
                                      /* 004:001, sz                       */
                                      /* 011:005, rsvd                     */
                                      /* 063:012, ptr                      */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 911: MMIO_ERR_CTL (MMIO Offset:0x2_0000) */
struct  SDXI_MMIO_ERR_CTL {
    uint64_t  intr_en;                /* 000:000, en                       */
                                      /* 063:001, rsvd                     */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 912: MMIO_ERR_STS (MMIO Offset:0x2_0008) */
struct  SDXI_MMIO_ERR_STS {
    uint64_t  info;                   /* 000:000, sts                      */
                                      /* 001:001, ovf                      */
                                      /* 002:002, rsvd                     */
                                      /* 003:003, err                      */
                                      /* 063:004, rsvd                     */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 913: MMIO_ERR_CFG (MMIO Offset:0x2_0010) */
struct  SDXI_MMIO_ERR_CFG {
    uint64_t  ptr;                    /* 000:000, en                       */
                                      /* 005:001, sz                       */
                                      /* 011:006, rsvd                     */
                                      /* 063:012, ptr                      */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 914: MMIO_ERR_WRT (MMIO Offset: 0x2_0020) */
struct  SDXI_MMIO_ERR_WRT {
    uint64_t  index;                  /* 063:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 915: MMIO_ERR_RD (MMIO Offset: 0x2_0028) */
struct  SDXI_MMIO_ERR_RD {
    uint64_t  index;                  /* 063:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 916: MMIO_MB_CTL (MMIO Offset:0x3_0000) */
struct  SDXI_MMIO_MB_CTL {
    uint64_t  intr_en;                /* 000:000, snd                      */
                                      /* 001:001, rcv                      */
                                      /* 063:002, rsvd                     */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 917: MMIO_MBS_CTL (MMIO Offset: 0x3_0010) */
struct  SDXI_MMIO_MBS_CTL {
    uint16_t  msg;                    /* 000:000, msg                      */
                                      /* 015:001, rsvd                     */
    uint16_t  tgt_vf;                 /* 031:016,                          */
    uint8_t   rsvd_0[4];              /* 063:032,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 918: MMIO_MBS_ST (MMIO Offset: 0x3_1000 to 0x3_2FFF) */
struct  SDXI_MMIO_MBS_ST {
    uint64_t  ack;                    /* 063:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 919: MMIO_MBR_ST (MMIO Offset: 0x3_3000 to 0x3_4FFF) */
struct  SDXI_MMIO_MBR_ST {
    uint64_t  rcv;                    /* 063:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(8);


/* From Table 924: MMIO_MBS (MMIO Offset: 0x6_0000) */
struct  SDXI_MMIO_MBS {
    uint8_t   msg[16];                /* 127:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(16);


/* From Table 925: MMIO_MBR (MMIO Offset: 0x7_0000) */
struct  SDXI_MMIO_MBR {
    uint8_t   msg[16];                /* 127:000,                          */
} _SDXI_PACKED _SDXI_ALIGN(16);

#endif /* SDXI_MMIO_H */
