/*
   BSD 3-Clause Software License
   
   Copyright (c) 2022, The Storage Networking Industry Association.
   
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are
   met:
   
   * Redistributions of source code must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   
   * Neither the name of The Storage Networking Industry Association (SNIA)
     nor the names of its contributors may be used to endorse or promote
     products derived from this software without specific prior written
     permission.
   
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
   PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
   OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
   Based on SNIA SDXI specification: 1.0
   Suitable for gcc and clang compilers only.
*/

#ifndef SDXI_RKEY_H
#define SDXI_RKEY_H
#include <stdint.h>

#define _SDXI_PACKED    __attribute__((__packed__))
#define _SDXI_ALIGN(x)  __attribute__ ((aligned (x)))


/* From Table 38: RKey Table Entry (RKEY_ENT) */
struct  SDXI_RKEY_ENT {
    uint16_t  intr_num;               /* 000:000, vl                       */
                                      /* 001:001, iv                       */
                                      /* 002:002, pv                       */
                                      /* 003:003, ste                      */
                                      /* 014:004, intr_num                 */
                                      /* 015:015, rsvd                     */
    uint16_t  req_sfunc;              /* 031:016,                          */
    uint32_t  pasid;                  /* 051:032, pasid                    */
                                      /* 061:052, rsvd                     */
                                      /* 063:062, ph                       */
    uint16_t  stag;                   /* 079:064,                          */
    uint8_t   rsvd_0[6];              /* 127:080,                          */
} _SDXI_PACKED _SDXI_ALIGN(16);

#endif /* SDXI_RKEY_H */
